/*
 * Copyright 2014 Peter Olsson
 * 
 * This file is part of Brum Brum Rally.
 * 
 * Brum Brum Rally is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Brum Brum Rally is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef INTERNET_H
#define INTERNET_H

#include "Network.h"
#include <vector>

enum ServerStatus
{
	STATUS_SERVER_NONE,
	STATUS_SERVER_NOT_FOUND,
	STATUS_SERVER_CONNECTION_FAIL,
	STATUS_SERVER_NEW_LIST
	
};

struct ServerPlayer
{
	Uint64 id;
	IPaddress connectAddress;
	Uint32 nextConnectionAttempt;
	Uint32 nextConnectionInterval;
};

struct ConnectedPlayer
{
	IPaddress address;
	Uint16 port;
};

class Internet : public Network
{
public:
	Internet();
	~Internet();
	virtual void step();
	virtual std::string name() const;
	std::string serverMessage(const std::string& data);
private:
	virtual void handleMessage(Uint8 msgType, IPaddress address);
	virtual void onConnectionTimeout(const IPaddress& addr);
	virtual void refreshAsNeeded();
	
	bool isConnected(Uint64 id) const;
	
	std::vector<ServerPlayer> playerList;
	std::size_t iPlayer;
	std::map<Uint64, ConnectedPlayer> idMap;
	
	Uint16 privatePort;
	Uint16 publicPort;
	Uint64 playerID;
	
	SDL_Thread* thread;
	SDL_mutex* mutex;
	SDL_cond* cond;
	
	bool refresh;
	bool stopThread;
	bool newListReady;
	Uint32 listTime;
	
	std::vector<ServerPlayer> newList;
	Uint64 newPlayerID;
	std::string newStatus;
	
	std::string serverHost;
	std::string serverPath;
	Uint16 serverPort;
	
	friend int server_thread(void*);
};

enum
{
	MSG_I_CONNECT = 200,
	MSG_I_CONNECT_ACK
};

#endif
